//=========================================================
//  MusE
//  Linux Music Editor
//  $Id: mpevent.h,v 1.5 2002/07/14 10:37:55 muse Exp $
//
//  (C) Copyright 1999-2002 Werner Schweer (ws@seh.de)
//=========================================================

#ifndef __MPEVENT_H__
#define __MPEVENT_H__

#include <map>

class MidiEvent;
class EvData;

//---------------------------------------------------------
//   MidiPlayEvent
//---------------------------------------------------------

class MidiPlayEvent {
      unsigned char _port, _channel, _type, _a, _b;
      EvData* edata;

   public:
      MidiPlayEvent();
      MidiPlayEvent(const MidiPlayEvent&);
      MidiPlayEvent(int p, int c, int t, int a, int b)
        : _port(p), _channel(c), _type(t), _a(a), _b(b) {
            edata = 0;
            }
      MidiPlayEvent(int p, const unsigned char* data, int len);
      MidiPlayEvent(int p, EvData* data);
      MidiPlayEvent& operator=(const MidiPlayEvent&);

      ~MidiPlayEvent();
      int port()    const      { return _port;    }
      int channel() const      { return _channel; }
      int type()    const      { return _type;    }
      void setType(int val)    { _type = val;     }
      void setPort(int val)    { _port = val;     }
      void setChannel(int val) { _channel = val;  }
      int dataA()   const      { return _a;       }
      int dataB()   const      { return _b;       }
      void setA(int val)       { _a = val;        }
      void setB(int val)       { _b = val;        }
      const unsigned char* data() const;
      int len() const;
      void dump() const;
      bool isNote() const      { return _type == 0x90; }
      bool isNoteOff() const {
            return (_type == 0x80)||(_type == 0x90 && _b == 0);
            }
      EvData* eventData() const { return edata; }
      };

//---------------------------------------------------------
//   MPEventList
//---------------------------------------------------------

struct MPEventList : public std::multimap<int, MidiPlayEvent, std::less<int> > {
      void add(int tick, int a, int b, int c, int d, int e) {
         std::multimap<int, MidiPlayEvent, std::less<int> >
           ::insert(std::pair<const int, MidiPlayEvent> (tick, MidiPlayEvent(a,b,c,d,e)));
            }
      void add(int tick, int port, EvData* ed) {
         std::multimap<int, MidiPlayEvent, std::less<int> >
           ::insert(std::pair<const int, MidiPlayEvent> (tick, MidiPlayEvent(port, ed)));
         }
      void add(int tick, const MidiEvent* e, int port, int channel);
      };

typedef MPEventList::iterator iMPEvent;
typedef MPEventList::const_iterator ciMPEvent;
typedef std::pair <iMPEvent, iMPEvent> MPEventRange;

#endif

